//======================================================================================//
// FixedFunction Shader                                                                 //
//                                                                                      //
// By Christian Bieber, AVC Team Bieber 2007                                            //
//                                                                                      //
// BASIERT auf folgendem ATI-BEISPIEL:                                                  //
// author:   Pedro V. Sander                                                            //
//           ATI Research, Inc.                                                         //
//           3D Application Research Group                                              //
//           email: psander@ati.com                                                     //
//                                                                                      //
// Description: A programmable shader that emulates the fixed function pipeline         //
//                                                                                      //
//======================================================================================//
//   (C) 2003 ATI Research, Inc.  All rights reserved.                                  //
//======================================================================================//

#define PI  3.14f     

//this file contains light, fog, and texture types
//#include "FixedFuncShader.fxh"
 
#define NUM_LIGHTS               4

#define LIGHT_TYPE_NONE          0
#define LIGHT_TYPE_POINT         1
#define LIGHT_TYPE_SPOT          2
#define LIGHT_TYPE_DIRECTIONAL   3
#define LIGHT_NUM_TYPES          4

#define FOG_TYPE_NONE            0
#define FOG_TYPE_EXP             1
#define FOG_TYPE_EXP2            2
#define FOG_TYPE_LINEAR          3
#define FOG_NUM_TYPES            4

#define TEX_TYPE_NONE            0
#define TEX_TYPE_CUBEMAP         1
#define TEX_NUM_TYPES            2

#define TEXGEN_TYPE_NONE                          0
#define TEXGEN_TYPE_CAMERASPACENORMAL             1
#define TEXGEN_TYPE_CAMERASPACEPOSITION           2
#define TEXGEN_TYPE_CAMERASPACEREFLECTIONVECTOR   3
#define TEXGEN_NUM_TYPES                          4

#define TEXOP_DISABLE                            0
#define TEXOP_SELECTARG1                         1
#define TEXOP_SELECTARG2                         2
#define TEXOP_MODULATE                           3
#define TEXOP_MODULATE2X                         4
#define TEXOP_MODULATE4X                         5
#define TEXOP_ADD                                6
#define TEXOP_ADDSIGNED                          7
#define TEXOP_ADDSIGNED2X                        8
#define TEXOP_SUBTRACT                           9
#define TEXOP_ADDSMOOTH                         10
#define TEXOP_BLENDDIFFUSEALPHA                 11
#define TEXOP_BLENDTEXTUREALPHA                 12
#define TEXOP_BLENDFACTORALPHA                  13
#define TEXOP_BLENDTEXTUREALPHAPM               14
#define TEXOP_BLENDCURRENTALPHA                 15
#define TEXOP_PREMODULATE                       16
#define TEXOP_MODULATEALPHA_ADDCOLOR            17
#define TEXOP_MODULATECOLOR_ADDALPHA            18
#define TEXOP_MODULATEINVALPHA_ADDCOLOR         19
#define TEXOP_MODULATEINVCOLOR_ADDALPHA         20
#define TEXOP_BUMPENVMAP                        21
#define TEXOP_BUMPENVMAPLUMINANCE               22
#define TEXOP_DOTPRODUCT3                       23
#define TEXOP_NUM_TYPES                         25

#define TEXTA_DIFFUSE                            0
#define TEXTA_CURRENT                            1
#define TEXTA_TEXTURE                            2
#define TEXTA_TFACTOR                            3
#define TEXTA_NUM_TYPES                          4

#define TEXTA_MOD_NONE                                      0
#define TEXTA_MOD_COMPLEMENT                                1
#define TEXTA_MOD_ALPHAREPLICATE                            2
#define TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE   3
#define TEXTA_MOD_NUM_TYPES                                 4
//#define iTexLevelOriginalData 3;


// Structs and variables with default values
float2 vTex0UsagePixels = {100, 100};
float2 vTex0UsageFactor = {1.0, 1.0};
float2 vTex0UsagePerPixelFactor = {1.0, 1.0};
float2 vTex0ScalingFactor = {1.0, 1.0};

float fShaderVar[10] = {0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f};
float fShaderVarB[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
int   iShaderVar[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
float4 colorShaderVar[5] = {
  float4(128.f/255.f, 128.f/255.f, 128.f/255.f, 1.f),
  float4(128.f/255.f, 128.f/255.f, 128.f/255.f, 1.f),
  float4(128.f/255.f, 128.f/255.f, 128.f/255.f, 1.f),
  float4(128.f/255.f, 128.f/255.f, 128.f/255.f, 1.f),
  float4(128.f/255.f, 128.f/255.f, 128.f/255.f, 1.f)
};

//bool bSpecular : register(b0) = false;
bool bSpecular = false;

bool bCurvilinearProjection = false;
//bool bCurvilinearProjection : register(b6) = false;

//bool bTweening : register(b6) = false;
//bool bTweening = false;
//float fTweenFactor = 0.f;

float4 vMaterialColor = float4(192.f/255.f, 128.f/255.f, 96.f/255.f, 1.f);
float fMaterialPower = 0.f;

float4 vAmbientColor = float4(128.f/255.f, 128.f/255.f, 128.f/255.f, 1.f);
float4 vSpecularColor = float4(128.f/255.f, 128.f/255.f, 128.f/255.f, 1.f);


//fog settings
int iFogType = FOG_TYPE_NONE;
float4 vFogColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
float fFogStart = 1.f;
float fFogEnd = 10.f;
float fFogDensity = 1.f;
bool bFogRange = false;
//bool bFogRange : register(b7) = false;
/**/

//int iTexType[3] = {TEX_TYPE_NONE, TEX_TYPE_NONE, TEX_TYPE_NONE};
int iTexGenType[3] = {TEXGEN_TYPE_NONE, TEXGEN_TYPE_NONE, TEXGEN_TYPE_NONE};

int iColorOpType[3] = {TEXOP_DISABLE, TEXOP_DISABLE, TEXOP_DISABLE};
int iColorOpArg1[3] = {TEXTA_DIFFUSE, TEXTA_DIFFUSE, TEXTA_DIFFUSE};
int iColorOpArg2[3] = {TEXTA_DIFFUSE, TEXTA_DIFFUSE, TEXTA_DIFFUSE};
//int iColorOpMod1[3] = {TEXTA_MOD_NONE, TEXTA_MOD_NONE, TEXTA_MOD_NONE};
//int iColorOpMod2[3] = {TEXTA_MOD_NONE, TEXTA_MOD_NONE, TEXTA_MOD_NONE};
int iAlphaOpType[3] = {TEXOP_DISABLE, TEXOP_DISABLE, TEXOP_DISABLE};
int iAlphaOpArg1[3] = {TEXTA_DIFFUSE, TEXTA_DIFFUSE, TEXTA_DIFFUSE};
int iAlphaOpArg2[3] = {TEXTA_DIFFUSE, TEXTA_DIFFUSE, TEXTA_DIFFUSE};
//int iAlphaOpMod1[3] = {TEXTA_MOD_NONE, TEXTA_MOD_NONE, TEXTA_MOD_NONE};
//int iAlphaOpMod2[3] = {TEXTA_MOD_NONE, TEXTA_MOD_NONE, TEXTA_MOD_NONE};

float fBumpEnvMat00 = 0.f;
float fBumpEnvMat10 = 0.f;
float fBumpEnvMat01 = 0.f;
float fBumpEnvMat11 = 0.f;
float fBumpEnvLScale = 1.f;
float fBumpEnvLOffset = 1.f;

//texture tex0;
sampler2D s_2D[5];
bool bTexSet[3] = {false, false, false};

struct CLight
{
   int iType;
   float3 vPos;
   float3 vDir;
   float4 vAmbient;
   float4 vDiffuse;
   float4 vSpecular;
   float  fRange;
   float3 vAttenuation; //1, D, D^2;
   float3 vSpot;        //cos(theta/2), cos(phi/2), falloff
};

CLight lights[NUM_LIGHTS];

int iNumLights = 0;

//transformation matrices
float4x4 matWorldViewProj  : WORLDVIEWPROJ;
float4x4 matWorldView      : WORLDVIEW;
float4x4 matWorld          : WORLD;
float4x4 matView           : VIEW;
float4x4 matProjection     : PROJECTION;
float4x4 matWorldViewIT;
float4x4 matViewIT;

float4 vWorldCameraPosition = {0, 0, -2, 0};

//function output structures
struct VS_OUTPUT
{
         float4 Pos           : POSITION;
         //float4 vPosition     : POSITION1;
         float3 N             : NORMAL;
         float4 Color         : COLOR0;
         float4 ColorSpec     : COLOR1;
         float2 Tex[3]        : TEXCOORD;
         float3 PosiFog       : TEXCOORD4;
         float3 V             : TEXCOORD3;
         //float3 L[NUM_LIGHTS] : TEXCOORD6;
         float  Fog           : FOG;
};

struct VS2_OUTPUT
{
         float4 Pos           : POSITION;
         //float4 vPosition     : POSITION1;
         //float3 N             : NORMAL;
         float4 Color         : COLOR0;
         float4 ColorSpec     : COLOR1;
         float2 Tex[3]        : TEXCOORD;
         //float3 PosiFog       : TEXCOORD4;
         //float3 V             : TEXCOORD5;
         //float3 L[NUM_LIGHTS] : TEXCOORD6;
         float  Fog           : FOG;
};

struct COLOR_PAIR
{
   float4 Color         : COLOR0;
   float4 ColorSpec     : COLOR1;
};

//-----------------------------------------------------------------------------
// Name: vs_main_VertexLights()
// Desc: The vertex shader MIT Vertex basiertem Licht...
//-----------------------------------------------------------------------------
VS_OUTPUT vs_main_VertexLights(
  float4 vPosition  : POSITION0,
  //float4 vPosition2 : POSITION1,
  float3 vNormal    : NORMAL0,
  //float3 vNormal2   : NORMAL1,
  //float2 tc         : TEXCOORD0,
  float2 tc[3]      : TEXCOORD0
)
{
  VS_OUTPUT Out = (VS_OUTPUT) 0;
  //Out.vPosition = vPosition;

  vNormal = normalize(vNormal);
  Out.Pos = mul(matWorldViewProj, vPosition);

  float3 PosiFog = mul(matWorldView, vPosition);           //position In View Space
  Out.N = normalize(mul((float3x3)matWorldViewIT, vNormal)); //normal In View Space
  Out.V = -normalize(PosiFog);                          //viewer

  //automatic texture coordinate generation
  for(int i = 0; i < 3; i++)
  {
   Out.Tex[i] = float4((2.f * dot(Out.V, Out.N) * Out.N - Out.V) * (iTexGenType[i] == TEXGEN_TYPE_CAMERASPACEREFLECTIONVECTOR)
        + Out.N * (iTexGenType[i] == TEXGEN_TYPE_CAMERASPACENORMAL)
        + PosiFog * (iTexGenType[i] == TEXGEN_TYPE_CAMERASPACEPOSITION),
         0);
   Out.Tex[i].xy += tc[i] * (iTexGenType[i] == TEXGEN_TYPE_NONE);
  }

  //light computation
        //Out.Color = vMaterialColor;//vAmbientColor;
  Out.Color = vAmbientColor;// + vMaterialColor;
  //Out.Color.a = 0;
  Out.ColorSpec = 0;



  return Out;
}



//---------------------------------------------------------------------------------------------------------------------------------------
// Pixelshader
//---------------------------------------------------------------------------------------------------------------------------------------


//---------------------------------------------------------------------------------------------------------------------------------------
// Funktionen fr Shader
//---------------------------------------------------------------------------------------------------------------------------------------

#define GradConversion 0.017453292519943294444269911519245                                                  //1.5707963267948965 * 100/90

//Texturkoordinaten auf used Texture clampen
float2 clamp2usedTex (float texCx_, float texCy_, float pixelsize_) {
   float2 texC_out;
   texC_out.x = clamp(texCx_, pixelsize_, vTex0UsageFactor.x-pixelsize_);
   texC_out.y = clamp(texCy_, pixelsize_, vTex0UsageFactor.y-pixelsize_);
   return texC_out;
}
 
//Simple berblendung
float4 dissolve(float faktor_, float4 colorA_, float4 colorB_) {
  faktor_ = saturate(faktor_);
  float4 dissolveColor = ((1.0f-faktor_)*colorA_)  +  (faktor_*colorB_);
  return dissolveColor;
}

//Simple berblendung ohne Saturate
float4 dissolveFactor(float faktor_, float4 colorA_, float4 colorB_) {
  float4 dissolveColor = ((1.0f-faktor_)*colorA_)  +  (faktor_*colorB_);
  return dissolveColor;
}

//Kachel-Rand 
float tileBump (float2 position_, float2 kachelpostopleft_, float2 kachelgroesse_, float2 bumpsize_, int pumpType_)  {
  float Kbump_left = position_.x - kachelpostopleft_.x;
  float Kbump_right = (position_.x - kachelgroesse_.x - kachelpostopleft_.x) *-1.0f;
  float Kbump_top = position_.y - kachelpostopleft_.y;
  float Kbump_bottom = (position_.y - kachelgroesse_.y - kachelpostopleft_.y) *-1.0f;
  float xfaktor = bumpsize_.x/bumpsize_.y;
  float yfaktor = bumpsize_.y/bumpsize_.x;
  float4 retbumbFaktor = 0.0f;
  if (Kbump_top   <  bumpsize_.y) { 
    if ((Kbump_top*xfaktor < Kbump_left) && (Kbump_top*xfaktor < Kbump_right))
      if (pumpType_ == 0)   
        retbumbFaktor = 1.0f;
      else
        retbumbFaktor = 1-Kbump_top/bumpsize_.y; 
  }
  if (Kbump_bottom   <  bumpsize_.y) { 
    if ((Kbump_bottom*xfaktor < Kbump_left) && (Kbump_bottom*xfaktor < Kbump_right))  
      if (pumpType_ == 0)   
        retbumbFaktor = -1.0f;
      else
        retbumbFaktor = (1-Kbump_bottom/bumpsize_.y)*-1.0f; 
  }
  if (Kbump_left   <  bumpsize_.x) { 
    if ((Kbump_left*yfaktor < Kbump_top) && (Kbump_left*yfaktor < Kbump_bottom)) 
      if (pumpType_ == 0)   
        retbumbFaktor = 0.5f;
      else
        retbumbFaktor = 1-Kbump_left/bumpsize_.x; 
  }
  if (Kbump_right   <  bumpsize_.x) { 
    if ((Kbump_right*yfaktor < Kbump_top) && (Kbump_right*yfaktor < Kbump_bottom)) 
      if (pumpType_ == 0)   
        retbumbFaktor = -0.5f;
      else
        retbumbFaktor = (1-Kbump_right/bumpsize_.x)*-1.0f; 
  }
  return retbumbFaktor; 
}

float2 posRotation (float2 position_, float2 center_, float rotation_) {
  float tmphypo = sqrt(((position_.x-center_.x)*(position_.x-center_.x)) + ((position_.y-center_.y)*(position_.y-center_.y))) ;
  if (position_.y-center_.y < 0.0f )
    tmphypo *= -1.0f;
  float tmpalpha = asin((position_.x-center_.x)/tmphypo) + rotation_;
    
  float2 tmppos;
  tmppos.x = center_.x + sin(tmpalpha)*tmphypo;   
  tmppos.y = center_.y + cos(tmpalpha)*tmphypo;  

  return tmppos;
}

struct PS_OUTPUT
{   float4 Color : COLOR0;
    //float  Depth  : DEPTH;
};

PS_OUTPUT ps_main (in VS_OUTPUT In): COLOR
{
  PS_OUTPUT Out;

  Out.Color = 0;//In.Color + In.ColorSpec;
  Out.Color.a = 1;

//      ColOut.Color = tex2D(s_2D[i], In.Tex[i]);  //if (iAlphaOpArg2[i] == TEXTA_TEXTURE), was meistens!!! der Fall ist wenn schon eine Textur gesetzt wurde^^...

  COLOR_PAIR ColOut;
  //float4 tColor1, tColor2;
  //for(int i = 0; i < 3; i++)
  int i = 0;
  {
    if (bTexSet[i])
    {
      ColOut.Color = tex2D(s_2D[i], In.Tex[i]);  //if (iAlphaOpArg2[i] == TEXTA_TEXTURE), was meistens!!! der Fall ist wenn schon eine Textur gesetzt wurde^^...
      //ColOut.ColorSpec = ColOut.Color;

      if (ColOut.Color.r > (1.0 - vMaterialColor.a))
      {
       Out.Color.a = 0;
      } else {
        i = 1;
        if (bTexSet[i])
        {
         Out.Color = vAmbientColor * tex2D(s_2D[i], In.Tex[i]);
         Out.Color.a =  1.0;
        } else {
          Out.Color = vAmbientColor;
        }
      }
    }
  }
  return Out;
}



//---------------------------------------------------------------------------------------------------------------------------------------
// PixelShader  ps 3.0
//---------------------------------------------------------------------------------------------------------------------------------------

PS_OUTPUT ps_mainFF(
   in VS_OUTPUT In
): COLOR
{
  PS_OUTPUT Out;

  Out.Color = In.Color + In.ColorSpec;

  COLOR_PAIR ColOut;
  //float4 tColor1, tColor2;
  for(int i = 0; i < 3; i++)
  //for(int i = 2; i >= 0; i--)
  {
    if (bTexSet[i])
    {
      ColOut.Color = tex2D(s_2D[i], In.Tex[i]);  //if (iAlphaOpArg2[i] == TEXTA_TEXTURE), was meistens!!! der Fall ist wenn schon eine Textur gesetzt wurde^^...
      ColOut.ColorSpec = ColOut.Color;
      /*
      ColOut.Color.rgb = (
        (iColorOpArg1[i] == TEXTA_DIFFUSE) * In.Color.rgb
        + (iColorOpArg1[i] == TEXTA_CURRENT) * Out.Color.rgb
        + (iColorOpArg1[i] == TEXTA_TEXTURE) * ColOut.Color.rgb
      );

      ColOut.ColorSpec.rgb = (
        (iColorOpArg2[i] == TEXTA_DIFFUSE) * In.Color.rgb
        + (iColorOpArg2[i] == TEXTA_CURRENT) * Out.Color.rgb
        + (iColorOpArg2[i] == TEXTA_TEXTURE) * ColOut.ColorSpec.rgb
      );

      ColOut.Color.a = (
        (iAlphaOpArg1[i] == TEXTA_DIFFUSE) * In.Color.a
        + (iAlphaOpArg1[i] == TEXTA_CURRENT) * Out.Color.a
        + (iAlphaOpArg1[i] == TEXTA_TEXTURE) * ColOut.Color.a
      );

      ColOut.ColorSpec.a = (
        (iAlphaOpArg2[i] == TEXTA_DIFFUSE) * In.Color.a
        + (iAlphaOpArg2[i] == TEXTA_CURRENT) * Out.Color.a
        + (iAlphaOpArg2[i] == TEXTA_TEXTURE) * ColOut.ColorSpec.a
      );
      /**/

      if (iColorOpArg1[i] == TEXTA_DIFFUSE) {ColOut.Color.rgb = In.Color.rgb;}
      if (iColorOpArg1[i] == TEXTA_CURRENT) {ColOut.Color.rgb = Out.Color.rgb;}
      //if (iColorOpArg1[i] == TEXTA_TEXTURE) {ColOut.Color.rgb = tex2D(s_2D[i], In.Tex[i]).rgb;}

      if (iColorOpArg2[i] == TEXTA_DIFFUSE) {ColOut.ColorSpec.rgb = In.Color.rgb;}
      if (iColorOpArg2[i] == TEXTA_CURRENT) {ColOut.ColorSpec.rgb = Out.Color.rgb;}
      //if (iColorOpArg2[i] == TEXTA_TEXTURE) {ColOut.ColorSpec.rgb = tex2D(s_2D[i], In.Tex[i]).rgb;}

      if (iAlphaOpArg1[i] == TEXTA_DIFFUSE) {ColOut.Color.a = In.Color.a;}
      if (iAlphaOpArg1[i] == TEXTA_CURRENT) {ColOut.Color.a = Out.Color.a;}
      //if (iAlphaOpArg1[i] == TEXTA_TEXTURE) {ColOut.Color.a = tex2D(s_2D[i], In.Tex[i]).a;}

      if (iAlphaOpArg2[i] == TEXTA_DIFFUSE) {ColOut.ColorSpec.a = In.Color.a;}
      if (iAlphaOpArg2[i] == TEXTA_CURRENT) {ColOut.ColorSpec.a = Out.Color.a;}
      //if (iAlphaOpArg2[i] == TEXTA_TEXTURE) {ColOut.ColorSpec.a = tex2D(s_2D[i], In.Tex[i]).a;}
      /**/

      /*
      if (
        (iColorOpMod1[i] == TEXTA_MOD_COMPLEMENT)
        || (iColorOpMod1[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
      )
      {ColOut.Color.rgb = 1 - ColOut.Color.rgb;}
      if (
        (iColorOpMod2[i] == TEXTA_MOD_COMPLEMENT)
        || (iColorOpMod2[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
      )
      {ColOut.ColorSpec.rgb = 1 - ColOut.ColorSpec.rgb;}

      if (
        (iAlphaOpMod1[i] == TEXTA_MOD_COMPLEMENT)
        || (iAlphaOpMod1[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
      )
      {ColOut.Color.a = 1 - ColOut.Color.a;}
      if (
        (iAlphaOpMod2[i] == TEXTA_MOD_COMPLEMENT)
        || (iAlphaOpMod2[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
      )
      {ColOut.ColorSpec.a = 1 - ColOut.ColorSpec.a;}
      /**/

      /*
      if (
        (iAlphaOpMod1[i] == TEXTA_MOD_ALPHAREPLICATE)
        || (iAlphaOpMod1[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
        || (iColorOpMod1[i] == TEXTA_MOD_ALPHAREPLICATE)
        || (iColorOpMod1[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
      )
      {
				ColOut.Color.r = ColOut.Color.a;
        ColOut.Color.g = ColOut.Color.a;
        ColOut.Color.b = ColOut.Color.a;
      }
      if (
        (iAlphaOpMod2[i] == TEXTA_MOD_ALPHAREPLICATE)
        || (iAlphaOpMod2[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
        || (iColorOpMod2[i] == TEXTA_MOD_ALPHAREPLICATE)
        || (iColorOpMod2[i] == TEXTA_MOD_COMPLEMENT_AND_D3DTA_MOD_ALPHAREPLICATE)
      )
      {
        ColOut.ColorSpec.r = ColOut.ColorSpec.a;
        ColOut.ColorSpec.g = ColOut.ColorSpec.a;
        ColOut.ColorSpec.b = ColOut.ColorSpec.a;
      }/**/

      if (iColorOpType[i] == TEXOP_MODULATE) {Out.Color.rgb = ColOut.Color.rgb * ColOut.ColorSpec.rgb;}
      if (iColorOpType[i] == TEXOP_MODULATE2X) {Out.Color.rgb = (ColOut.Color.rgb * ColOut.ColorSpec.rgb) * 2;}
      if (iColorOpType[i] == TEXOP_MODULATE4X) {Out.Color.rgb = (ColOut.Color.rgb * ColOut.ColorSpec.rgb) * 4;}
      if (iColorOpType[i] == TEXOP_ADD) {Out.Color.rgb = ColOut.Color.rgb + ColOut.ColorSpec.rgb;}
      if (iColorOpType[i] == TEXOP_ADDSIGNED) {Out.Color.rgb = ColOut.Color.rgb + ColOut.ColorSpec.rgb - 0.5;}
      if (iColorOpType[i] == TEXOP_ADDSIGNED2X) {Out.Color.rgb = (ColOut.Color.rgb + ColOut.ColorSpec.rgb - 0.5) * 2;}
      if (iColorOpType[i] == TEXOP_SUBTRACT) {Out.Color.rgb = ColOut.Color.rgb - ColOut.ColorSpec.rgb;}
      /*if (
        (iColorOpType[i] == TEXOP_BUMPENVMAP)
        || (iColorOpType[i] == TEXOP_BUMPENVMAPLUMINANCE)
      )
      {
        int j = min(2, i+1);
        In.Tex[j].x += (ColOut.Color.r * fBumpEnvMat00) + (ColOut.Color.g * fBumpEnvMat10);
        In.Tex[j].y += (ColOut.Color.r * fBumpEnvMat01) + (ColOut.Color.g * fBumpEnvMat11);

        if (iColorOpType[i] == TEXOP_BUMPENVMAPLUMINANCE)
          Out.Color *= fBumpEnvLOffset + (fBumpEnvLScale * ColOut.Color.b);
      }
      if (iColorOpType[i] == TEXOP_DOTPRODUCT3)
      {
        Out.Color.a = dot(ColOut.Color.rgb, ColOut.ColorSpec.rgb);
        Out.Color.r = Out.Color.a;
        Out.Color.g = Out.Color.a;
        Out.Color.b = Out.Color.a;
      }/**/

      if (iAlphaOpType[i] == TEXOP_MODULATE) {Out.Color.a = ColOut.Color.a * ColOut.ColorSpec.a;}
      if (iAlphaOpType[i] == TEXOP_MODULATE2X) {Out.Color.a = (ColOut.Color.a * ColOut.ColorSpec.a) * 2;}
      if (iAlphaOpType[i] == TEXOP_MODULATE4X) {Out.Color.a = (ColOut.Color.a * ColOut.ColorSpec.a) * 4;}
			if (iAlphaOpType[i] == TEXOP_ADD) {Out.Color.a = ColOut.Color.a + ColOut.ColorSpec.a;}
      if (iAlphaOpType[i] == TEXOP_ADDSIGNED) {Out.Color.a = ColOut.Color.a + ColOut.ColorSpec.a - 0.5;}
      if (iAlphaOpType[i] == TEXOP_ADDSIGNED2X) {Out.Color.a = (ColOut.Color.a + ColOut.ColorSpec.a - 0.5) * 2;}
      if (iAlphaOpType[i] == TEXOP_SUBTRACT) {Out.Color.a = ColOut.Color.a - ColOut.ColorSpec.a;}
      /**/
      //{Out.Color.rgb = Out.Color.rgb * tex2D(s_2D[i], In.Tex[i]).rgb;}
    }
  };

  Out.Color = saturate(Out.Color + In.ColorSpec);


  // Fog...
  float d; // = length(In.PosiFog);

  if(bFogRange)
    d = length(In.PosiFog);
  else
    d = In.PosiFog.z;

  d = 1.f * (iFogType == FOG_TYPE_NONE)
    + 1.f/exp(d * fFogDensity) * (iFogType == FOG_TYPE_EXP)
    + 1.f/exp(pow(d * fFogDensity, 2)) * (iFogType == FOG_TYPE_EXP2)
    //+ saturate((fFogEnd - d)/(fFogEnd - fFogStart)) * (iFogType == FOG_TYPE_LINEAR);
    + saturate((fFogEnd - d)/(fFogEnd - fFogStart)) * (iFogType == FOG_TYPE_LINEAR);

  Out.Color.rgb = lerp(
    vFogColor.rgb,
    Out.Color.rgb,
    d //In.Fog
  );
  /**/

  return Out;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// Blur ps3.0    
//---------------------------------------------------------------------------------------------------------------------------------------
// AVC floats
#define fDEF_blur_dissolveFactor 0
#define fDEF_blur_radius 1
#define fDEF_blur_radius_x 2
#define fDEF_blur_radius_y 3
// other defines
#define nDEF_blur_pixel 16
PS_OUTPUT AVCps3_blur_ex_x (in VS_OUTPUT In): COLOR
{
  PS_OUTPUT Out;
  float tmp1f = 0.0f;
  float tmp2f = 0.0f;
  float BlurFaktor = fShaderVar[fDEF_blur_radius] + fShaderVar[fDEF_blur_radius_x];
  Out.Color = 0.0f;   
    
//  if (bTexSet[1]) 
//    BlurFaktor *=  tex2D(s_2D[1], In.Tex[1]).r;
  
  for (int i = 0; i < nDEF_blur_pixel; i++) {
    Out.Color += tex2D(s_2D[0], float2((In.Tex[0].x/vTex0UsageFactor.x) + (BlurFaktor*0.0051f * (tmp1f-0.5001f)), 
                                       In.Tex[0].y/vTex0UsageFactor.y))
                * 4.0f/float(nDEF_blur_pixel)                                                 			      //Ausgleich auf Anzahl der beeinflussenden Pixel.
                * tmp2f;												   //Abschwchung des Pixeleinflusses in Abhngikeit von der Entfernung zum Mittelpunkt.
    				
    tmp1f += 1.0f/float(nDEF_blur_pixel); 
    if (tmp1f > 0.5f)	
	  tmp2f -= 1.0f/float(nDEF_blur_pixel);	  
	else
	  tmp2f += 1.0f/float(nDEF_blur_pixel); 
  }  
  
  return Out;
}
PS_OUTPUT AVCps3_blur_ex_y (in VS_OUTPUT In): COLOR
{
  PS_OUTPUT Out;
  float tmp1f = 0.0f;
  float tmp2f = 0.0f;
  float BlurFaktor = fShaderVar[fDEF_blur_radius] + fShaderVar[fDEF_blur_radius_y];
  Out.Color = 0.0f;   
  
 // if (bTexSet[1]) 
 //   BlurFaktor *= tex2D(s_2D[1], In.Tex[1]).g;
  
  for (int i = 0; i < nDEF_blur_pixel; i++) {
    Out.Color += tex2D(s_2D[0], float2(In.Tex[0].x, 
                                   (In.Tex[0].y + (BlurFaktor*0.0051f * (tmp1f-0.5001f))) ))  
                * 4.0f/float(nDEF_blur_pixel)                                                 			      //Ausgleich auf Anzahl der beeinflussenden Pixel.
                * tmp2f;												   //Abschwchung des Pixeleinflusses in Abhngikeit von der Entfernung zum Mittelpunkt.
    				
    tmp1f += 1.0f/float(nDEF_blur_pixel); 
    if (tmp1f > 0.5f)	
	  tmp2f -= 1.0f/float(nDEF_blur_pixel);	  
	else
	  tmp2f += 1.0f/float(nDEF_blur_pixel); 
  }  
  

  //berblendung mit dem Original-Bild (TexLevelFirstPassData = 3)
  Out.Color = dissolve(fShaderVar[fDEF_blur_dissolveFactor], tex2D(s_2D[3], In.Tex[0]), Out.Color);
  
 //Out.Color = tex2D(s_2D[3], In.Tex[0]);
 
  //Einblendung mit dem Color/Alpha-Befehl
  Out.Color *= vAmbientColor;

  return Out;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// Blur_X ps3.0    
//---------------------------------------------------------------------------------------------------------------------------------------
// AVC floats
#define fDEF_blurX_dissolveFactor 0
#define fDEF_blurX_radius 1
// other defines
#define nDEF_blurX_pixel 24
PS_OUTPUT AVCps3_blurX_ex (in VS_OUTPUT In): COLOR
{
  PS_OUTPUT Out;
  float tmp1f = 0.0f;
  float tmp2f = 0.0f;
  float BlurFaktor = fShaderVar[fDEF_blurX_radius];
  Out.Color = 0.0f;   
    
  if (bTexSet[1]) 
    BlurFaktor *=  tex2D(s_2D[1], In.Tex[1]).r;
  
  for (int i = 0; i < nDEF_blurX_pixel; i++) {
    Out.Color += tex2D(s_2D[0], float2((In.Tex[0].x/vTex0UsageFactor.x) + (BlurFaktor*0.0051f * (tmp1f-0.5001f)), 
                                       In.Tex[0].y/vTex0UsageFactor.y))
                * 4.0f/float(nDEF_blurX_pixel)                                                 			      //Ausgleich auf Anzahl der beeinflussenden Pixel.
                * tmp2f;												   //Abschwchung des Pixeleinflusses in Abhngikeit von der Entfernung zum Mittelpunkt.
    				
    tmp1f += 1.0f/float(nDEF_blurX_pixel); 
    if (tmp1f > 0.5f)	
	  tmp2f -= 1.0f/float(nDEF_blurX_pixel);	  
	else
	  tmp2f += 1.0f/float(nDEF_blurX_pixel); 
  }  
    
  //berblendung mit dem Original-Bild
  Out.Color = dissolve(fShaderVar[fDEF_blurX_dissolveFactor], tex2D(s_2D[3], In.Tex[0]), Out.Color);

  //Einblendung mit dem Color/Alpha-Befehl
  Out.Color.rgba *= vAmbientColor;

  
  return Out;
}
//---------------------------------------------------------------------------------------------------------------------------------------
// Blur_Y ps3.0    
//---------------------------------------------------------------------------------------------------------------------------------------
// AVC floats
#define fDEF_blurY_dissolveFactor 0
#define fDEF_blurY_radius 1
// other defines
#define nDEF_blurY_pixel 24
PS_OUTPUT AVCps3_blurY_ex (in VS_OUTPUT In): COLOR
{
  PS_OUTPUT Out;
  float tmp1f = 0.0f;
  float tmp2f = 0.0f;
  float BlurFaktor = fShaderVar[fDEF_blurY_radius];
  Out.Color = 0.0f;   
    
  if (bTexSet[1]) 
    BlurFaktor *=  tex2D(s_2D[1], In.Tex[1]).r;
  
  for (int i = 0; i < nDEF_blurY_pixel; i++) {
    Out.Color += tex2D(s_2D[0], float2((In.Tex[0].x/vTex0UsageFactor.x) + (BlurFaktor*0.0051f * (tmp1f-0.5001f)), 
                                       In.Tex[0].y/vTex0UsageFactor.y))
                * 4.0f/float(nDEF_blurY_pixel)                                                 			      //Ausgleich auf Anzahl der beeinflussenden Pixel.
                * tmp2f;												   //Abschwchung des Pixeleinflusses in Abhngikeit von der Entfernung zum Mittelpunkt.
    				
    tmp1f += 1.0f/float(nDEF_blurY_pixel); 
    if (tmp1f > 0.5f)	
	  tmp2f -= 1.0f/float(nDEF_blurY_pixel);	  
	else
	  tmp2f += 1.0f/float(nDEF_blurY_pixel); 
  }  
    
  //berblendung mit dem Original-Bild
  Out.Color = dissolve(fShaderVar[fDEF_blurY_dissolveFactor], tex2D(s_2D[3], In.Tex[0]), Out.Color);

  //Einblendung mit dem Color/Alpha-Befehl
  Out.Color.rgba *= vAmbientColor;

  
  return Out;
}
//---------------------------------------------------------------------------------------------------------------------------------------
// Blur Radialzoom ps3.0
//---------------------------------------------------------------------------------------------------------------------------------------
// AVC floats
#define fDEF_blur_radialzoom_dissolveFactor 0
#define fDEF_blur_radialzoom_effect 1
#define fDEF_blur_radialzoom_center_x 2
#define fDEF_blur_radialzoom_center_y 3
// other defines
#define nDEF_blur_radialzoom_pixel 16
PS_OUTPUT AVCps3_blur_radialzoom_ex (in VS_OUTPUT In): COLOR
{
  PS_OUTPUT Out;
  
  float tmp1f;
  Out.Color.rgba = 0.0;   
  float2 center;
  center.x = (vTex0UsageFactor.x/2.0f) + fShaderVar[fDEF_blur_radialzoom_center_x];
  center.y = (vTex0UsageFactor.y/2.0f) + fShaderVar[fDEF_blur_radialzoom_center_y];
  
  for (int i = 0; i < nDEF_blur_radialzoom_pixel; i++) {
    Out.Color += tex2D(s_2D[0], clamp2usedTex(((In.Tex[0].x-center.x) / (1.0f + (fShaderVar[fDEF_blur_radialzoom_effect] * (1.0f-(float(i)/float(nDEF_blur_radialzoom_pixel-1)))))) + center.x, 
                                              ((In.Tex[0].y-center.y) / (1.0f + (fShaderVar[fDEF_blur_radialzoom_effect] * (1.0f-(float(i)/float(nDEF_blur_radialzoom_pixel-1)))))) + center.y,
                                                0.002f)) 
               * ((float(i)/float(nDEF_blur_radialzoom_pixel-1))/float(nDEF_blur_radialzoom_pixel)) * 2.0f;  //Faktor fr langsam abfallende Auswirkung  (Er macht einen Durchlauf umsonst wenn i=0 ist, da das Ergebnis dann 0 ist. Dafr stimmt der Helligkeitswert mit dem ogignalBild berein! Und es muss kein krumme Konstante als Ausgleich verwendet werden.)
  } 
       
  //berblendung mit Maske  
  if (bTexSet[1]) 
    tmp1f = fShaderVar[fDEF_blur_radialzoom_dissolveFactor]  * (tex2D(s_2D[1], In.Tex[1]).g);   //berblendung mit "Mask-texture 1"
  else
    tmp1f = fShaderVar[fDEF_blur_radialzoom_dissolveFactor];
    
  //berblendung mit dem Original-Bild
  Out.Color = dissolve(tmp1f, tex2D(s_2D[3], In.Tex[0]), Out.Color);

  //Einblendung mit dem Color/Alpha-Befehl
  Out.Color.rgba *= vAmbientColor;

  return Out;
}



//---------------------------------------------------------------------------------------------------------------------------------------
// Blur Spinmotion ps3.0
//---------------------------------------------------------------------------------------------------------------------------------------
// AVC floats
#define fDEF_blur_spinmotion_dissolveFactor 0
#define fDEF_blur_spinmotion_rotation 1
#define fDEF_blur_spinmotion_center_x 2
#define fDEF_blur_spinmotion_center_y 3
#define fDEF_blur_spinmotion_distortion_x 4 //Muss noch wie "vTex0ScalingFactor" eingerechnet werden VIELLICHT ...
#define fDEF_blur_spinmotion_distortion_y 5 //Muss noch wie "vTex0ScalingFactor" eingerechnet werden VIELLICHT ...
#define fDEF_blur_spinmotion_borderwidth 6 //Noch nicht umgesetzt!!!! 
// other defines
#define nDEF_blur_spinmotion_pixel 16
// AVC colors
#define cDEF_blur_spinmotion_borderColor 0
PS_OUTPUT AVCps3_blur_spinmotion_ex (in VS_OUTPUT In): COLOR
{
  PS_OUTPUT Out;
  
fShaderVar[fDEF_blur_spinmotion_borderwidth] = 1.1f;


  float tmp1f;
  Out.Color = 0.0f;
  float2 center;
  center.x = (vTex0UsageFactor.x/2.0f) + fShaderVar[fDEF_blur_spinmotion_center_x];
  center.y = (vTex0UsageFactor.y/2.0f) + fShaderVar[fDEF_blur_spinmotion_center_y];
  int tmp = sqrt(nDEF_blur_spinmotion_pixel);
  float newalpha;
  float hypothenuse;
  
In.Tex[0].x /= vTex0ScalingFactor.x;  ////ACHTUNG: Nichtquadratische Textur-Anpassung !!! BETA!!!!!!!!!!!!!!!!!!!
center.x /= vTex0ScalingFactor.x;  ////ACHTUNG: Nichtquadratische Textur-Anpassung !!! BETA!!!!!!!!!!!!!!!!!!!
In.Tex[0].y /= vTex0ScalingFactor.y;  ////ACHTUNG: Nichtquadratische Textur-Anpassung !!! BETA!!!!!!!!!!!!!!!!!!!
center.y /= vTex0ScalingFactor.y;  ////ACHTUNG: Nichtquadratische Textur-Anpassung !!! BETA!!!!!!!!!!!!!!!!!!!
//vTex0UsageFactor.x /= vTex0ScalingFactor.x;
//vTex0UsageFactor.y /= vTex0ScalingFactor.y;

  //Winkel zum Mittelpunkt  
  hypothenuse = sqrt(((In.Tex[0].x-center.x)*(In.Tex[0].x-center.x))  +   ((In.Tex[0].y-center.y)*(In.Tex[0].y-center.y))) ;
  if ((In.Tex[0].y-center.y < 0 ))
    hypothenuse *= -1.0f;
  for (int i = 0; i < nDEF_blur_spinmotion_pixel; i++) {
    newalpha = asin((In.Tex[0].x-center.x)/hypothenuse)  +  ((float(i)/float(nDEF_blur_spinmotion_pixel-1)) * GradConversion * fShaderVar[fDEF_blur_spinmotion_rotation]);
    Out.Color += tex2D(s_2D[0], clamp2usedTex((center.x + (sin(newalpha)*hypothenuse)) * vTex0ScalingFactor.x, 
                                          (center.y + (cos(newalpha)*hypothenuse)) * vTex0ScalingFactor.y,  ////ACHTUNG: Nichtquadratische Textur-Anpassung !!! BETA!!!!!!!!!!!!!!!!!!!
                                         0.002f*fShaderVar[fDEF_blur_spinmotion_borderwidth]))
               * 1.0f/nDEF_blur_spinmotion_pixel;
  }
  

In.Tex[0].x *= vTex0ScalingFactor.x;  ////ACHTUNG: Nichtquadratische Textur-Anpassung !!! BETA!!!!!!!!!!!!!!!!!!!
In.Tex[0].y *= vTex0ScalingFactor.y;  ////ACHTUNG: Nichtquadratische Textur-Anpassung !!! BETA!!!!!!!!!!!!!!!!!!!
  
 //berblendung mit Maske  
  if (bTexSet[1]) 
    tmp1f = fShaderVar[fDEF_blur_spinmotion_dissolveFactor]  * (tex2D(s_2D[1], In.Tex[1]).g);   //berblendung mit "Mask-texture 1"
  else
    tmp1f = fShaderVar[fDEF_blur_spinmotion_dissolveFactor];
    
  //berblendung mit dem Original-Bild
  Out.Color = dissolve(tmp1f, tex2D(s_2D[3], In.Tex[0]), Out.Color);

  //Einblendung mit dem Color/Alpha-Befehl
  Out.Color *= vAmbientColor;
  
  return Out; 
}



//---------------------------------------------------------------------------------------------------------------------------------------
// Techniques  ps 3.0
//---------------------------------------------------------------------------------------------------------------------------------------

technique AVCfx3_blur3pass
// AVC floats
// AVC: fShaderVar[0] Dissolve-factor
// AVC: fShaderVar[1] Blur-radius
// AVC: fShaderVar[2] Blur-radius_x
// AVC: fShaderVar[3] Blur-radius_y
{ pass P0
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 ps_mainFF();
  }/**/
  pass P1
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blur_ex_x();
  }
	pass P2
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blur_ex_y();
  }/**/
}

/*
technique AVCfx3_blur2pass
// AVC floats
// AVC: fShaderVar[0] Dissolve-factor
// AVC: fShaderVar[1] Blur-radius
// AVC: fShaderVar[2] Blur-radius_x
// AVC: fShaderVar[3] Blur-radius_y
{ pass P0
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blur_ex_x();
  }
  pass P1
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blur_ex_y();
  }
}/**/

technique AVCfx3_blur_x
// AVC floats
// AVC: fShaderVar[0] Dissolve-factor
// AVC: fShaderVar[1] Blur-radius_x
{ 
	pass P0
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 ps_mainFF();
  }/**/
	pass P1
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blurX_ex();
  }
}
technique AVCfx3_blur_y
// AVC floats
// AVC: fShaderVar[0] Dissolve-factor
// AVC: fShaderVar[1] Blur-radius_y
{ 
	pass P0
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 ps_mainFF();
  }/**/
	pass P1
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blurY_ex();
  }
}


technique AVCfx3_blur_radialzoom
// AVC floats
// AVC: fShaderVar[0] Dissolve-factor
// AVC: fShaderVar[1] Radialzoom-factor
// AVC: fShaderVar[2] Radialzoom-center_x
// AVC: fShaderVar[3] Radialzoom-center_y
{ 
	pass P0
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 ps_mainFF();
  }/**/
	pass P1
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blur_radialzoom_ex();
  }
} 

technique AVCfx3_blur_spinmotion
// AVC floats
// AVC: fShaderVar[0] Dissolve-factor
// AVC: fShaderVar[1] Spin-rotation
// AVC: fShaderVar[2] Spin-center_x
// AVC: fShaderVar[3] Spin-center_y
// AVC: fShaderVar[4] Spin-distortion_x
// AVC: fShaderVar[5] Spin-distortion_y
// AVC colors
// AVC: colorShaderVar[0] Spin-border-color
{ 
	pass P0
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 ps_mainFF();
  }/**/
	pass P1
  { VertexShader = compile vs_3_0 vs_main_VertexLights();
    PixelShader = compile ps_3_0 AVCps3_blur_spinmotion_ex();
  }
}

